/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@padova.infn.it                        |
 | WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html    |
 *------------------------------------------------------*

 config.h: include file for config.c (see comments there
           for explanations)

 $Id: config.h,v 1.4 1999/09/29 12:39:23 loreti Exp $

 *------------------------------------------------------*/

#ifndef CONFIG_H_
#define CONFIG_H_

#include <stdio.h>              /* FILE */
#include <stdarg.h>

/**
 | Shortcuts
**/

#define CONFIG_ERROR    -1
#define CONFIG_OK        0

#define CONFIG_COMMENT  ';'
#define CONFIG_EQUAL    '='
#define CONFIG_QUOTE    '\\'
#define CONFIG_SEP      ":"

/**
 | The maximum line length in a configuration file; since an array
 | of strings must be on a single line, the same constant will be
 | used also to allocate a temporary buffer for string arrays.
**/

#define CONFIG_LINE_LEN         128

/**
 | This macro checks if the (char *) returned by the standard library
 | procedures strto{l|ul|d} points to whitespace; if not, the procedure
 | has stopped on an invalid character.
**/

#define LAST_FIELD(p)   (isspace((unsigned char) *p)   ||   *p == '\0')

/**
 | Possible types for the variables to be configured
**/

typedef enum eConfigType {
  CONFIG_LONG,     CONFIG_ULONG,
  CONFIG_INT,      CONFIG_UINT,
  CONFIG_DOUBLE,   CONFIG_STRING,
  CONFIG_STRARRAY
} cType;

/**
 | An entry (in a double-linked list) for every variable to be
 | configured:
 | - two pointers (prev and next) to the neighbour nodes;
 | - a pointer (target) to the variable in memory;
 | - an auxilliary pointer (auxTarget) to be used for string
 |   arrays;
 | - the dimension of the target (num), if it is a vector;
 | - the variable type (type) and the associated symbolic
 |   keyword (name).
**/

typedef struct sConfigNode {
  struct sConfigNode  *prev;
  struct sConfigNode  *next;
  void                *target;
  char               **auxTarget;
  unsigned int         num;
  cType                type;
  char                 name[1];
} cNode;

/**
 | Procedure prototypes
**/

int addConfig(const int, cType, const char *, unsigned int, void *, ...);
int readConfig(const int, FILE *, const char *);
int remConfig(const int);

#endif /* CONFIG_H_ */
