/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@padova.infn.it                        |
 | WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html    |
 *------------------------------------------------------*
 
 keyword.h: include file for keyword.c (see comments
             there for explanations)
 
 $Id: keyword.h,v 1.2 1999/04/29 08:33:20 loreti Exp $
 
 *------------------------------------------------------*/

#ifndef KEYWORD_H_
#define KEYWORD_H_

/**
 | Error values from initKeyTable and getKeyword
**/

typedef enum eKeyErr {
  KE_OK = 0,                    /* All OK                            */
  KE_UNDEF,                     /* Undefined keyword (getKeyword)    */
  KE_AMBIG,                     /* Ambiguous keyword (both routines) */
  KE_DUPL,                      /* Duplicate keyword (initKeyTable)  */
  KE_NOMEM,                     /* Error from malloc (initKeyTable)  */
  KE_INVDATA                    /* Invalid input (both routines)     */
} KeyErr;

/**
 | Data structures for the keyword table and a keyword entry
**/

typedef struct sKey {
  size_t n;                     /* Number of characters to match */
  int userdata;                 /* User-defined data             */
  char *id;                     /* Pointer to the keyword        */
} Key;

typedef struct sKeyTable {
  size_t n;                     /* Number of keywords            */
  Key *start;                   /* Key[n] array to store entries */
} KeyTable;

/**
 | Prototypes
**/

KeyTable *initKeyTable(size_t n, char *key[], int user[], KeyErr *error);
int getKeyword(KeyTable *table, char *string, KeyErr *error);
void releaseKeyTable(KeyTable *table);

#endif /* KEYWORD_H_ */
